#
# nasd_apt.awk - write records with colon delimited fields from apt.txt
#
# Copyright(C) 1999-2000, John C. Peterson <mailto:jaypee@netcom.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2. A copy is included in this distribution in the file
# named "LICENSE".
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License, version 2, for more details.
#
# $Id: nasd_apt.awk,v 1.1 2000/02/29 04:57:09 jcp Exp $
#

# helper function to extract a sub string and strip leading and trailing blanks

function pack_substr(s, i, n)
{
  # fetch sub string
  string = substr(s, i, n);
  # strip leading blanks
  sub(/^ */, "", string);
  # strip trailing blanks
  sub(/ *$/, "", string);
  # all done
  return string;
}

BEGIN { facility_site_no = ""; }

/^APT/ {
  # L AN 0011 00004  DLID    LANDING FACILITY SITE NUMBER
  facility_site_no = pack_substr($0, 4, 11);

  # L AN 0013 00015  DLID    LANDING FACILITY TYPE
  #                             AIRPORT
  #                             BALLOONPORT
  #                             SEAPLANE BASE
  #                             GLIDERPORT
  #                             HELIPORT
  #                             STOLPORT
  #                             ULTRALIGHT
  facility_type = pack_substr($0, 15, 13);

  # L AN 0004 00028  E7      LOCATION IDENTIFIER
  facility_ident = pack_substr($0, 28, 4);

  # L AN 0002 00049  A4      ASSOCIATED STATE POST OFFICE CODE
  state_code = pack_substr($0, 49, 2);

  # L AN 0020 00051  A4      ASSOCIATED STATE NAME
  state_name = pack_substr($0, 51, 20);

  # L AN 0026 00094  A1      ASSOCIATED CITY NAME
  city_name = pack_substr($0, 94, 26);
  gsub(/  +/, " ", city_name);

  # L AN 0042 00120  A2      OFFICIAL FACILITY NAME
  facility_name = pack_substr($0, 120, 42);
  gsub(/  +/, " ", facility_name);

  # L AN 0002 00162  A10     AIRPORT OWNERSHIP TYPE
  #                             PU - PUBLICLY OWNED
  #                             PR - PRIVATELY OWNED
  #                             MA - AIR FORCE OWNED
  #                             MN - NAVY OWNED
  #                             MR - ARMY OWNED
  facility_ownership = pack_substr($0, 162, 2);

  # L AN 0002 00164  A18     FACILITY USE
  #                             PU - OPEN TO THE PUBLIC
  #                             PR - PRIVATE
  facility_usage = pack_substr($0, 164, 2);

  # L AN 0014 00364  A19     AIRPORT REFERENCE POINT LATITUDE (FORMATTED)
  latitude = pack_substr($0, 364, 14);

  # L AN 0014 00389  A20     AIRPORT REFERENCE POINT LONGITUDE (FORMATTED)
  longitude = pack_substr($0, 389, 14);

  # L AN 0001 00414  A19A    AIRPORT REFERENCE POINT DETERMINATION METHOD
  #                             E - ESTIMATED
  #                             S - SURVEYED
  survey_accuracy = pack_substr($0, 414, 1);

  # R AN 0005 00415  A21     AIRPORT ELEVATION  (NEAREST FOOT MSL)
  #                             ELEVATION IS MEASURED AT THE HIGHEST POINT
  #                             ON THE CENTERLINE OF THE USABLE LANDING SURFACE
  #                             (EX. 1200; -10 FOR 10 FEET BELOW SEA LEVEL)
  airport_elevation = pack_substr($0, 415, 5);

  # L AN 0003 00421  E28     MAGNETIC VARIATION AND DIRECTION
  mag_variation = pack_substr($0, 421, 3);

  # L AN 0004 00424  E28     MAGNETIC VARIATION EPOCH YEAR
  mag_variation_epoch_year = pack_substr($0, 424, 4);

  # R AN 0004 00428  E147    TRAFFIC PATTERN ALTITUDE (WHOLE FEET AGL)
  traffic_pattern_altitude = pack_substr($0, 428, 4);

  # L AN 0016 00432  A7      AERONAUTICAL SECTIONAL CHART ON WHICH FACILITY
  #                          APPEARS. (EX. WASHINGTON)
  aeronautical_sectional_chart = pack_substr($0, 432, 16);
  gsub(/  +/, " ", aeronautical_sectional_chart);

  # L AN 0004 00506  A86     TIE-IN FLIGHT SERVICE STATION (FSS) IDENTIFIER
  #                             (EX. DCA FOR WASHINGTON FSS)
  flight_service_station_ident = pack_substr($0, 506, 4);

  # L AN 0026 00510  A86     TIE-IN FSS NAME
  #                             (EX. WASHINGTON)
  flight_service_station_name = pack_substr($0, 510, 26);

  # L AN 0016 00536  A89     TOLL FREE PHONE NUMBER FROM AIRPORT TO FSS
  #                          FOR PILOT BRIEFING SERVICES
  flight_service_station_phone_number = pack_substr($0, 536, 16);

  # L AN 0040 00687  A70     FUEL TYPES AVAILABLE FOR PUBLIC USE AT THE
  #                          AIRPORT. THERE CAN BE UP TO 8 OCCURENCES
  #                          OF A FIXED 5 CHARACTER FIELD.
  #                          (EX. 80___100__100LL115__)
  #
  #                             80    - GRADE 80 GASOLINE (RED)
  #                             100   - GRADE 100 GASOLINE (GREEN)
  #                             100LL - GRADE 100LL GASOLINE (LOW LEAD BLUE)
  #                             115   - GRADE 115 GASOLINE
  #                             A     - JET A - KEROSENE, FREEZE POINT -40C
  #                             A1    - JET A-1 - KEROSENE, FREEZE POINT -50C
  #                             A1+   - JET A-1 - KEROSENE, WITH ICING INHIBITOR
  #                                     FREEZE POINT -50C
  #                             B     - JET B - WIDE-CUT TURBINE FUEL, FREEZE
  #                                     POINT -50C
  #                             B+    - JET B - WIDE-CUT TURBINE FUEL WITH ICING
  #                                     INHIBITOR, FREEZE POINT -50C
  #                             MOGAS - AUTOMOTIVE GASOLINE
  #
  fuel_types_available_for_public_use = pack_substr($0, 687, 40);
  sub(/100LL/, "100LL ", fuel_types_available_for_public_use);
  sub(/MOGAS/, "MOGAS ", fuel_types_available_for_public_use);
  sub(/^ */, "", fuel_types_available_for_public_use);
  sub(/ *$/, "", fuel_types_available_for_public_use);
  gsub(/ +/, "/", fuel_types_available_for_public_use);

  # L AN 0005 00727  A71     AIRFRAME REPAIR SERVICE AVAILABILITY/TYPE
  #                             MAJOR
  #                             MINOR
  #                             NONE
  frame_repair_service_availability = pack_substr($0, 727, 5);

  # L AN 0005 00732  A72     POWER PLANT (ENGINE) REPAIR AVAILABILITY/TYPE
  #                             MAJOR
  #                             MINOR
  #                             NONE
  engine_repair_service_availability = pack_substr($0, 732, 5);

  # L AN 0008 00737  A73     TYPE OF BOTTLED OXYGEN AVAILABLE (VALUE REPRESENTS
  #                          HIGH AND/OR LOW PRESSURE REPLACEMENT BOTTLE)
  #                             HIGH
  #                             LOW
  #                             HIGH/LOW
  #                             NONE
  bottled_oxygen_availability = pack_substr($0, 737, 8);
  sub(/HIGH/, "HI", bottled_oxygen_availability);
  sub(/LOW/, "LO", bottled_oxygen_availability);

  # L AN 0008 00745  A74     TYPE OF BULK OXYGEN AVAILABLE (VALUE REPRESENTS
  #                          HIGH AND/OR LOW PRESSURE CYLINDERS)
  #                             HIGH
  #                             LOW
  #                             HIGH/LOW
  #                             NONE
  bulk_oxygen_availability = pack_substr($0, 745, 8);
  sub(/HIGH/, "HI", bulk_oxygen_availability);
  sub(/LOW/, "LO", bulk_oxygen_availability);

  # L AN 0077 00753  A17     AIRPORT ATTENDANCE SCHEDULE (WHEN MINIMUM
  #                          SERVICES ARE AVAILABLE)
  facility_attendance_schedule = pack_substr($0, 753, 77);

  # L AN 0007 00882  E100    COMMON TRAFFIC ADVISORY FREQUENCY (CTAF)
  #                             (EX. 122.800)
  common_traffic_advisory_frequency = pack_substr($0, 882, 7);

  # write a colon delimited record
  #   field 1 *must* be the record type
  #   field 2 *must* be the facility identifier

  printf "APT:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s\n",
    facility_ident,
    facility_name,
    facility_type,
    facility_ownership,
    facility_usage,
    facility_attendance_schedule,
    state_code,
    state_name,
    city_name,
    latitude,
    longitude,
    survey_accuracy,
    airport_elevation,
    mag_variation,
    mag_variation_epoch_year,
    common_traffic_advisory_frequency,
    traffic_pattern_altitude,
    aeronautical_sectional_chart,
    flight_service_station_ident,
    flight_service_station_name,
    flight_service_station_phone_number,
    fuel_types_available_for_public_use,
    frame_repair_service_availability,
    engine_repair_service_availability,
    bottled_oxygen_availability,
    bulk_oxygen_availability;
}

/^RWY/ {
  # L AN 0011 00004  DLID    LANDING FACILITY SITE NUMBER
  #                             THE UNIQUE IDENTIFYING NUMBER OF THE
  #                             AIRPORT WHOSE RUNWAY IS BEING DESCRIBED.
  #                             TOGETHER WITH THE RUNWAY ID FIELD, THIS
  #                             PROVIDES THE UNIQUE KEY TO A RUNWAY RECORD.
  runway_site_no = pack_substr($0, 4, 11);

  # L AN 0007 00017  DLID    RUNWAY IDENTIFICATION
  #                             EX. 01/19; 18L/36R (PARALLEL RUNWAYS);
  #                                 H1 (HELIPAD); N/S (NORTH/SOUTH);
  #                                 ALL/WAY (SEALANE); B1 (BALLOONPORT)
  runway_ident = pack_substr($0, 17, 7);

  # R AN 0005 00024  A31     PHYSICAL RUNWAY LENGTH (NEAREST FOOT)
  #                            (EX. 3500)
  runway_length = pack_substr($0, 24, 5);

  # R AN 0004 00029  A32     PHYSICAL RUNWAY WIDTH (NEAREST FOOT)
  #                            (EX. 100)
  runway_width = pack_substr($0, 29, 4);

  # L AN 0012 00033  A33     RUNWAY SURFACE TYPE AND CONDITION
  #                            CONC    - PORTLAND CEMENT CONCRETE
  #                            ASPH    - ASPHALT OR BITUMINOUS CONCRETE
  #                            SNOW    - SNOW
  #                            ICE     - ICE
  #                            MATS    - PIERCED STEEL PLANKING; LANDING
  #                                      MATS; MEMBRANES
  #                            TREATED - OILED; SOIL CEMENT OR LIME
  #                                      STABILIZED
  #                            GRAVEL  - GRAVEL; CINDERS; CRUSHED ROCK;
  #                                      CORAL OR SHELLS; SLAG
  #                            TURF    - GRASS; SOD
  #                            DIRT    - NATURAL SOIL
  #                            WATER   - WATER
  #                            G       - GOOD CONDITION
  #                            F       - FAIR CONDITION
  #                            P       - POOR CONDITION
  #
  #                          THE VALUE CAN BE ANY ONE OF THOSE DESCRIBED
  #                          ABOVE OR A COMBINATION OF TWO TYPES WHEN
  #                          THE RUNWAY IS COMPOSED OF DISTINCT SECTIONS.
  #
  #                          THE SURFACE TYPE IS OPTIONALLY FOLLOWED BY
  #                          A CONDITION INDICATOR.
  #                          (EX. ASPH-CONC TURF-GRVL ASPH-F ASPH-CONC-G)
  runway_surface = pack_substr($0, 33, 12);

  # check for match of site numbers with last airport record

  if (runway_site_no != facility_site_no) next;

  # write a colon delimited record
  #   field 1 *must* be the record type
  #   field 2 *must* be the facility identifier

  printf "RWY:%s:%s:%s:%s:%s\n",
    facility_ident,
    runway_ident,
    runway_length,
    runway_width,
    runway_surface;
}
