#
# nasd_fix.awk - write records with colon delimited fields from fix.txt
#
# Copyright(C) 1999-2000, John C. Peterson <mailto:jaypee@netcom.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2. A copy is included in this distribution in the file
# named "LICENSE".
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License, version 2, for more details.
#
# $Id: nasd_fix.awk,v 1.1 2000/02/29 04:57:09 jcp Exp $
#

# helper function to extract a sub string and strip leading and trailing blanks

function pack_substr(s, i, n)
{
  # fetch sub string
  string = substr(s, i, n);
  # strip leading blanks
  sub(/^ */, "", string);
  # strip trailing blanks
  sub(/ *$/, "", string);
  # all done
  return string;
}

{
  # L  AN  30 00001  NONE    RECORD IDENTIFIER (FIX NAME)
  fix_name = pack_substr($0, 1, 30);

  # L  AN0020 00031  NONE    RECORD IDENTIFIER (FIX STATE NAME)
  state_name = pack_substr($0, 31, 20);

  # L  AN0014 00051  FX4     GEOGRAPHICAL LATITUDE OF THE FIX.
  latitude = pack_substr($0, 51, 14);

  # L  AN0014 00065  FX5     GEOGRAPHICAL LONGITUDE OF THE FIX.
  longitude = pack_substr($0, 65, 14);

  # L  AN0003 00079  FX20    CATEGORIZES THE FIX AS A MILITARY, CIVIL,
  #                          OR CARIBBEAN FIX OR AS A WAYPOINT
  fix_category = pack_substr($0, 79, 3);

  # L  AN0022 00082  FX21    FIRST CHART ON WHICH FIX IS TO BE DEPICTED
  #                          EX: IAP, AREA, CONTROLLER, MILITARY IAP,
  #                              MILITARY SID, MILITARY STAR, NOT REQUIRED,
  #                              SPECIAL IAP, STAR, SID, RNAV HIGH,
  #                              RNAV LOW, ENROUTE LOW, ENROUTE HIGH,
  #                              IFR GOM VERTICAL FLT, HELICOPTER ROUTE,
  #                              SECTIONAL, VFR TERMINAL AREA,
  #                              VFR FLYWAY PLANNING, CONTROLLER CHART ONLY,
  #                              PRIVATE IAP, PROFILE DESCENT
  fix_chart = pack_substr($0, 82, 22);

  # L  AN0022 00302  FX22    LOCATION-IDENTIFIER, FACILITY TYPE, AND RADIAL
  #                          OR BEARING / DME DISTANCE OF FIRST NAVAID USED
  #                          IN THE FIX DESCRIPTION
  loc_id_facility_type_radial_dist_navaid_1 = pack_substr($0, 302, 22);
  gsub(/\*/, " ", loc_id_facility_type_radial_dist_navaid_1);

  # L  AN0022 00324  FX23    LOCATION-IDENTIFIER, FACILITY TYPE, AND RADIAL
  #                          OR BEARING / DME DISTANCE OF SECOND NAVAID USED
  #                          IN THE FIX DESCRIPTION
  loc_id_facility_type_radial_dist_navaid_2 = pack_substr($0, 324, 22);
  gsub(/\*/, " ", loc_id_facility_type_radial_dist_navaid_2);

  # L  AN0022 00346  FX24    LOCATION-IDENTIFIER, FACILITY TYPE, AND RADIAL
  #                          OR BEARING / DME DISTANCE OF THIRD NAVAID USED
  #                          IN THE FIX DESCRIPTION
  loc_id_facility_type_radial_dist_navaid_3 = pack_substr($0, 346, 22);
  gsub(/\*/, " ", loc_id_facility_type_radial_dist_navaid_2);

  # L  AN0022 00368  FX25    LOCATION-IDENTIFIER, FACILITY TYPE, AND RADIAL
  #                          OR BEARING / DME DISTANCE OF FOURTH NAVAID USED
  #                          IN THE FIX DESCRIPTION
  loc_id_facility_type_radial_dist_navaid_4 = pack_substr($0, 368, 22);
  gsub(/\*/, " ", loc_id_facility_type_radial_dist_navaid_4);

  # L  AN0022 00390  FX26    3 OR 4 LETTER IDENT*FACILITY TYPE*DIRECTION OR
  #                          COURSE OF FIRST ILS OR MLS COMPONENT USED IN FIX
  #                          DESCRIPTION
  loc_id_facility_type_dir_ils_mls_in_fix_1 = pack_substr($0, 390, 22);
  gsub(/\*/, " ", loc_id_facility_type_dir_ils_mls_in_fix_1);

  # L  AN0022 00412  FX27    3 OR 4 LETTER IDENT*FACILITY TYPE*DIRECTION OR
  #                          COURSE OF SECOND ILS OF MLS COMPONENT USED IN FIX
  #                          DESCRIPTION
  loc_id_facility_type_dir_ils_mls_in_fix_2 = pack_substr($0, 412, 22);
  gsub(/\*/, " ", loc_id_facility_type_dir_ils_mls_in_fix_2);

  # L  AN0015 00586  FX36    FIX USE
  #                          EX.  ARTCC-BDRY (ARTCC BOUNDARY)
  #                               AWY-INTXN (AIRWAY INTERSECTION)
  #                               COORDN-FIX (ATC COORDINATION FIX)
  #                               MIL-REP-PT (MILITARY REPORTING POINT)
  #                               MIL-WAYPOINT (MILITARY WAYPOINT)
  #                               REP-PT (REPORTING POINT)
  #                               TURN-PT (TURNING POINT)
  #                               WAYPOINT (WAYPOINT)
  fix_use = pack_substr($0, 586, 15);

  # L  AN0005 00601  FX37    NATIONAL AIRSPACE SYSTEM(NAS) IDENTIFIER FOR
  #                          THE FIX (USUALLY 5 LETTER OR 5 NUMBER).
  #                    NOTE: IF THE FIX OVERLAYES A NAVAID AND HAS AN ID
  #                          WHICH IS NOT 5 CHARACTERS, THEN FX37 WILL DISPLAY
  #                          THE LOCATION IDENTIFIER OF THE NAVAID.
  fix_ident = pack_substr($0, 601, 5);

  # L  AN0003 00606  FX91    DENOTES ARTCC AREA OF JURISDICTION.
  artcc_area_of_jurisdiction = pack_substr($0, 606, 3);

  # L  AN0020 00609  NONE    FIX COUNTRY NAME (OUTSIDE CONUS)
  country_name = pack_substr($0, 609, 20);

  # write a colon delimited record
  #   field 1 *must* be the record type
  #   field 2 *must* be the facility identifier

  printf "FIX:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s\n",
    fix_ident,
    fix_name,
    fix_category,
    fix_use,
    fix_chart,
    artcc_area_of_jurisdiction,
    state_name,
    country_name,
    latitude,
    longitude,
    loc_id_facility_type_radial_dist_navaid_1,
    loc_id_facility_type_radial_dist_navaid_2,
    loc_id_facility_type_radial_dist_navaid_3,
    loc_id_facility_type_radial_dist_navaid_4,
    loc_id_facility_type_dir_ils_mls_in_fix_1,
    loc_id_facility_type_dir_ils_mls_in_fix_2;
}
