#
# nasd_nav.awk - write records with colon delimited fields from nav.txt
#
# Copyright(C) 1999-2000, John C. Peterson <mailto:jaypee@netcom.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2. A copy is included in this distribution in the file
# named "LICENSE".
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License, version 2, for more details.
#
# $Id: nasd_nav.awk,v 1.1 2000/02/29 04:57:09 jcp Exp $
#

# helper function to extract a sub string and strip leading and trailing blanks

function pack_substr(s, i, n)
{
  # fetch sub string
  string = substr(s, i, n);
  # strip leading blanks
  sub(/^ */, "", string);
  # strip trailing blanks
  sub(/ *$/, "", string);
  # all done
  return string;
}

/^NAV1/ {
  # L AN 0004 00005  DLID    NAVAID FACILITY IDENTIFIER
  facility_ident = pack_substr($0, 5, 4);

  # L AN 0020 00009  DLID    NAVAID FACILITY TYPE (SEE DESCRIPTION)
  #                          (EX: VOR/DME)
  #                  NAVAID           NAVAID
  # 
  #                            TYPE          DESCRIPTION
  #                            ----          ------------
  #                            VORTAC        A FACILITY CONSISTING OF TWO
  #                                          COMPONENTS, VOR AND TACAN,
  #                                          WHICH PROVIDES THREE INDIVIDUAL
  #                                          SERVICES: VOR AZIMUTH, TACAN
  #                                          AZIMUTH AND TACAN DISTANCE(DME)
  #                                          AT ONE SITE.
  # 
  #                            VOR/DME       VHF OMNI-DIRECTIONAL RANGE WITH
  #                                          ASSOCIATED DISTANCE MEASURING
  #                                          EQUIPMENT
  # 
  #                            FAN MARKER    THERE ARE 3 TYPES OF EN ROUTE
  #                                          MARKER BEACONS. FAN MARKER
  #                                          LOW POWERED FAN MARKERS AND
  #                                          Z MARKERS.
  #                                          A FAN MARKER IS USED TO PROVIDE
  #                                          A POSITIVE IDENTIFICATION OF
  #                                          POSITIONS AT DEFINITE POINTS
  #                                          ALONG THE AIRWAYS.
  # 
  #                            CONSOLAN      A LOW FREQUENCY, LONG-DISTANCE
  #                                          NAVAID USED PRINCIPALLY FOR
  #                                          TRANSOCEANIC NAVIGATION.
  # 
  #                            MARINE NDB    A NON DIRECTIONAL BEACON
  #                                          USED PRIMARILY FOR MARINE
  #                                          (SURFACE) NAVIGATION.
  # 
  #                            MARINE NDB/DME A NON DIRECTINAL BEACON
  #                                           WITH ASSOCIATED DISTANCE
  #                                           MEASURING EQUIPMENT; USED
  #                                           PRIMARILY FOR MARINE (SURFACE)
  #                                           NAVIGATION.
  # 
  #                            VOT           A FAA VOR TEST FACILITY
  # 
  #                            NDB           A NONDIRECTIONAL BEACON
  # 
  #                            NDB/DME       NON DIRECTIONAL BEACON WITH
  #                                          ASSOCIATED DISTANCE MEASURING
  #                                          EQUIPMENT.
  # 
  #                            TACAN         A TACTICAL AIR NAVIGATION
  #                                          SYSTEM PROVIDING ASIMUTH AND
  #                                          SLANT RANGE DISTANCE.
  # 
  #                            UHF/NDB       ULTRA HIGH FREQUENCY/
  #                                          NON DIRECTIONAL BEACON
  # 
  #                            VOR           A VHF OMNI-DIRECTIONAL RANGE
  #                                          PROVIDENCE ASIMUTH ONLY
  # 
  facility_type = pack_substr($0, 9, 20);

  # L AN 0026 00043  N8      NAME OF NAVAID.(EX: WASHINGTON)
  facility_name = pack_substr($0, 43, 26);

  # L AN 0026 00069  N1      CITY ASSOCIATED WITH THE NAVAID.
  #                          (EX: WASHINGTON)
  city_name = pack_substr($0, 69, 26);

  # L AN 0020 00095  N2      STATE NAME WHERE ASSOCIATED CITY IS LOCATED
  state_name = pack_substr($0, 95, 20);

  # L AN 0002 00115  N2S     STATE POST OFFICE CODE WHERE ASSOCIATED
  #                          CITY IS LOCATED.
  state_code = pack_substr($0, 115, 2);

  # L AN 0020 00120  N3      COUNTRY NAVAID LOCATED IF OTHER THAN U.S
  #                          (NAME) (EX: CANADA)
  country_name = pack_substr($0, 120, 20);

  # L AN 0002 00140  N3S     COUNTRY POST OFFICE CODE NAVAID
  #                          LOCATED IF OTHER THAN U.S. (EX: CA)
  country_code = pack_substr($0, 140, 2);

  # L AN 0011 00244  N28     CLASS OF NAVAID.
  #                          THE NAVAID CLASS DESIGNATOR MAY BE COMPRISED
  #                          OF AN ALTITUDE CODE (VOR, VORTAC, VOR/DME, AND
  #                          TACAN FACILITIES ONLY), AND/OR A COMBINATION
  #                          OF CLASS CODES.
  # 
  # 
  #                                    _______________________
  # 
  #                          ALTITUDE CODE      ALTITUDE DESCRIPTION
  #                          -------------      --------------------
  #                          H                  HIGH
  #                          L                  LOW
  #                          T                  TERMINAL
  # 
  #                           SEPARATOR CHARACTER:  - (DASH)
  #                           -------------------
  # 
  #                               CLASS CODE/DESCRIPTION
  #                               ----------------------
  #                          AB     AUTOMATIC WEATHER BROADCAST.
  # 
  #                          DME    UHF STANDARD (TACAN COMPATIBLE) DISTANCE
  #                                 MEASURING EQUIPMENT.
  # 
  #                          DME(Y) UHF STANDARD (TACAN COMPATIBLE) DISTANCE
  #                                 MEASURING EQUIPMENT THAT REQUIRES TACAN
  #                                 RECEIVERS TO BE PLACED IN THE 'Y' MODE
  #                                 TO RECEIVE DME
  # 
  #                          H      NON-DIRECTIONAL RADIO BEACON (NDB),
  #                                 (HOMING), POWER 50 WATTS TO LESS THAN
  #                                 2000 WATTS (50 NM AT ALL ALTITUDES).
  # 
  #                          HH     NON-DIRECTIONAL RADIO BEACON (NDB),
  #                                 (HOMING), POWER 2000 WATTS OR MORE
  #                                 (75 NM AT ALL ALTITUDES)
  # 
  #                          H-SAB  NON-DIRECTIONAL RADIO BEACON PROVIDING
  #                                 AUTOMATIC TRANSCRIBED WEATHER SERVICE.
  # 
  #                          LMM    COMPASS LOCATOR STATION WHEN INSTALLED
  #                                 AT MIDDLE MARKER SITE (15 NM AT ALL
  #                                 ALTITUDES).
  # 
  #                          LOM    COMPASS LOCATOR STATION WHEN INSTALLED
  #                                 AT OUTER MARKER SITE (15 NM AT ALL
  #                                 ALTITUDES).
  # 
  #                          MH     NON-DIRECTIONAL RADIO BEACON (NDB)
  #                                 (HOMING), POWER LESS THAN 50 WATTS
  #                                 (25 NM AT ALL ALTITUDES)
  # 
  #                          S      SIMULTANEOUS RANGE HOMING SIGNAL AND/OR
  #                                 VOICE
  # 
  #                          SABH   NON-DIRECTIONAL RADIO BEACON (NDB) NOT
  #                                 AUTHORIZED FOR IFR OR ATC. PROVIDES
  #                                 AUTOMATIC WEATHER BROADCASTS.
  # 
  #                          TACAN  UHF NAVIGATIONAL FACILITY-OMNIDIREC-
  #                                 TIONAL COURSE AND DISTANCE INFORMATION.
  # 
  #                          VOR    VHF NAVIGATIONAL FACILITY-OMNIDIREC-
  #                                 TIONAL COURSE ONLY.
  # 
  #                          VOR/DME COLLOCATED VOR NAVIGATIONAL FACILITY
  #                                  AND UHF STANDARD DISTANCE MEASURING
  #                                  EQUIPMENT.
  # 
  #                          VORTAC  COLLOCATED VOR AND TACAN NAVIGATIONAL
  #                                  FACILITIES.
  # 
  #                          W       WITHOUT VOICE ON RADIO FACILITY
  #                                  FREQUENCY.
  # 
  #                          Z       VHF STATION LOCATION MARKER AT A LF
  #                                  RADIO FACILITY.
  # 
  #                          EXAMPLES:   H-ABVORTAC, L-VOR, H, HH, MH-SAB,
  #                                      MHW/LOM, H-SAB/LOM
  # 
  #                          NOTE:       MULTIPLE CLASS CODE TYPES MAY BE
  #                                      SEPARATED BY A / (SLANT) OR A - (DASH)
  # 
  #                          **** AUXILIARY CANADA CLASS CODES   ****
  #                                    --------------------
  # 
  #                          THESE CODES MAY APPEAR SINGLY, IN
  #                          MULTIPLES, OR COMBINED WITH THE CODES LISTED
  #                          ABOVE:
  # 
  #                                 CLASS CODE/DESCRIPTION
  #                                 ----------------------
  # 
  #                          A       ATIS(AUTOMATIC TERMINAL INFORMATION
  #                                       SERVICE)
  # 
  #                          C       TRANSCRIBED WEATHER BROADCAST STATION
  # 
  #                          B       SCHEDULED WEATHER BROADCAST
  # 
  #                          T       FSS OR OTHER ATC AGENCY (EXCEPT PAR)
  #                                  CAN TRANSMIT ON THIS NAVIGATION FRE-
  #                                  QUENCY BUT NOT RECEIVE
  # 
  #                          P       PRECISION APPROACH RADAR BACK-UP FRE-
  #                                  QUENCY
  # 
  #                          L       NDB POWER OUTPUT LESS THAN 50 WATTS
  # 
  #                          M       NDB POWER OUTPUT 50 TO LESS THAN 2000
  #                                  WATTS
  # 
  #                          H       NDB POWER OUTPUT 2000 WATTS OR MORE
  # 
  #                          Z       75 MHZ STATION LOCATION MARKER OR FAN
  #                                  MARKER
  # 
  #                          EXAMPLE: M,L,CTMZ,TPM,MZ,LZ,TMZ,BT,
  #                                   TACAN,VOR/DME,VOR
  facility_class = pack_substr($0, 244, 11);

  # L AN 0009 00255  N52     HOURS OF OPERATION OF NAVAID
  #                           (EX:  0800-2400)
  facility_hours_of_operation = pack_substr($0, 255, 9);

  # L AN 0014 00284  N4      NAVAID LATITUDE (FORMATTED)
  latitude = pack_substr($0, 284, 14);

  # L AN 0014 00309  N5      NAVAID LONGITUDE (FORMATTED)
  longitude = pack_substr($0, 309, 14);

  # R AN 0005 00385  N37     ELEVATION IN  WHOLE FEET (MSL).
  elevation_feet = pack_substr($0, 385, 5);

  # R AN 0005 00390  N45     MAGNETIC VARIATION DEGREES   (00-99)
  #                          FOLLOWED BY MAGNETIC VARIATION
  #                          DIRECTION (E,W)
  #                           (EX: 8080W)
  magnetic_variation = pack_substr($0, 390, 5);

  # R AN 0004 00395  N45S    MAGNETIC VARIATION EPOCH YEAR  (00-99)
  magnetic_variation_epoch_year = pack_substr($0, 395, 4);

  # L  N 0004 00434 N24      CHANNEL (TACAN)  NAVAID TRANSMITS ON
  #                           (EX : 051X)
  transmit_tacan_channel = pack_substr($0, 434, 4);

  # R  N 0006 00438 N25      FREQUENCY THE NAVAID TRANSMITS ON
  #                           (EXCEPT TACAN)
  #                           (EX:  110.60 298)
  transmit_frequency = pack_substr($0, 438, 6);

  # L AN 0004 00517  N49S    ASSOCIATED/CONTROLLING FSS (IDENT)
  controlling_fss_ident = pack_substr($0, 517, 4);

  # L AN 0026 00521  N49     ASSOCIATED/CONTROLLING FSS (NAME)
  controlling_fss_name = pack_substr($0, 521, 26);

  # L AN 0100 00547  F15     HOURS OF OPERATION OF CONTROLLING FSS
  controlling_fss_hours_of_operation = pack_substr($0, 547, 100);

  # write a colon delimited record
  #   field 1 *must* be the record type
  #   field 2 *must* be the facility identifier

  printf "NAV:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s:%s\n",
    facility_ident,
    facility_name,
    facility_type,
    facility_class,
    facility_hours_of_operation,
    city_name,
    state_name,
    state_code,
    country_name,
    country_code,
    latitude,
    longitude,
    elevation_feet,
    magnetic_variation,
    magnetic_variation_epoch_year,
    transmit_tacan_channel,
    transmit_frequency,
    controlling_fss_ident,
    controlling_fss_name,
    controlling_fss_hours_of_operation;
}
