#
# nasd_twr.awk - write records with colon delimited fields from twr.txt
#
# Copyright(C) 1999-2000, John C. Peterson <mailto:jaypee@netcom.com>
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License,
# version 2. A copy is included in this distribution in the file
# named "LICENSE".
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License, version 2, for more details.
#
# $Id: nasd_twr.awk,v 1.1 2000/02/29 04:57:09 jcp Exp $
#

# helper function to extract a sub string and strip leading and trailing blanks

function pack_substr(s, i, n)
{
  # fetch sub string
  string = substr(s, i, n);
  # strip leading blanks
  sub(/^ */, "", string);
  # strip trailing blanks
  sub(/ *$/, "", string);
  # all done
  return string;
}

# define helper function to give priority to VHF frequencies

function select_vhf(freq_val, freq_str)
{
  # parse the numeric portion only
  if (match(freq_str,/[0-9]+\.[0-9]*/) > 0) {
    freq_num = substr(freq_str, RSTART, RLENGTH);
    # test for VHF frequency
    if (freq_num < vhf_max) {
      return freq_num;
    } else {
      if (freq_val == "") {
        return freq_num;
      } else {
        return freq_val;
      }
    }
  } else {
    # parse failed
    return freq_val;
  }
}

# define helper function to write a colon delimited record

function write_record()
{
  # write a colon delimited record
  #   field 1 *must* be the record type
  #   field 2 *must* be the facility identifier

  if (last_ident != "") {
    printf "TWR:%s:%s:%s:%s:%s:%s\n",
      last_ident,
      tower_freq_1,
      tower_freq_2,
      ground_freq_1,
      ground_freq_2,
      atis_freq;
  }

  tower_freq_1 = "";
  tower_freq_2 = "";
  ground_freq_1 = "";
  ground_freq_2 = "";
  atis_freq = "";
}

BEGIN { last_ident = ""; vhf_max = 150.0; }

/^TWR3/ {
  facility_ident = pack_substr($0, 5, 4);
  frequency[1] = pack_substr($0,   9, 44);
  frequency[2] = pack_substr($0, 103, 44);
  frequency[3] = pack_substr($0, 197, 44);
  frequency[4] = pack_substr($0, 291, 44);
  frequency[5] = pack_substr($0, 385, 44);
  frequency[6] = pack_substr($0, 479, 44);
  frequency[7] = pack_substr($0, 573, 44);
  frequency[8] = pack_substr($0, 667, 44);
  frequency[9] = pack_substr($0, 761, 44);
  frequency_sector[1] = pack_substr($0,  53, 50);
  frequency_sector[2] = pack_substr($0, 147, 50);
  frequency_sector[3] = pack_substr($0, 241, 50);
  frequency_sector[4] = pack_substr($0, 335, 50);
  frequency_sector[5] = pack_substr($0, 429, 50);
  frequency_sector[6] = pack_substr($0, 523, 50);
  frequency_sector[7] = pack_substr($0, 617, 50);
  frequency_sector[8] = pack_substr($0, 711, 50);
  frequency_sector[9] = pack_substr($0, 805, 50);

  # if this is a new identifier, write the last record

  if (facility_ident != last_ident) write_record();

  # check all fields for frequencies of interest

  for (i = 1; i < 10; i++) {
    if (frequency_sector[i] == "LCL/P") {
      tower_freq_1 = select_vhf(tower_freq_1, frequency[i]);
    } else if (frequency_sector[i] == "LCL/S") {
      tower_freq_2 = select_vhf(tower_freq_2, frequency[i]);
    } else if (frequency_sector[i] == "GND/P") {
      ground_freq_1 = select_vhf(ground_freq_1, frequency[i]);
    } else if (frequency_sector[i] == "GND/S") {
      ground_freq_2 = select_vhf(ground_freq_2, frequency[i]);
    } else if (frequency_sector[i] == "ATIS") {
      atis_freq = select_vhf(atis_freq, frequency[i]);
    }
  }

  # save the facility identifier

  last_ident = facility_ident;
}

END { write_record(); }
